@@ -5,4 +5,5 @@ from base import NodeModel
 from relationships import Relationship
 from properties import Property, StringProperty, EmailProperty,\
         URLProperty,IntegerProperty, DateProperty, DateTimeProperty,\
-        ArrayProperty, StringArrayProperty, IntArrayProperty, URLArrayProperty
+        ArrayProperty, StringArrayProperty, IntArrayProperty,\
+        URLArrayProperty, AutoProperty
