@@ -12,11 +12,13 @@ from django.utils.encoding import force_unicode
 from neo4jrestclient.client import NotFoundError
 
 from neo4django.decorators import transactional
-from base import NodeModel
-from relationships import Relationship
+from .base import NodeModel
+from .relationships import Relationship
+from .. import connections
 from neo4django.validators import validate_array, validate_str_array,\
         validate_int_array, ElementValidator
 from neo4django.utils import AttrRouter, write_through
+from neo4django.constants import AUTO_PROP_INDEX_VALUE
 
 MIN_INT = -9223372036854775808
 MAX_INT = 9223372036854775807
@@ -35,22 +37,29 @@ class Property(object):
     }
 
     def __init__(self, indexed=False, indexed_fulltext=False,
-                 indexed_range=False, has_own_index = False, unique=False,
-                 name=None, editable=True, null=True, blank=True, validators=[],
-                 choices=None, error_messages=None, required=False,
-                 serialize=True, metadata={}, default=NOT_PROVIDED,
-                 **kwargs):
+                 indexed_range=False, indexed_by_member=False,
+                 has_own_index=False, unique=False, name=None, editable=True,
+                 null=True, blank=True, validators=[], choices=None,
+                 error_messages=None, required=False, serialize=True,
+                 auto=False, metadata={}, auto_default=NOT_PROVIDED, 
+                 default=NOT_PROVIDED, **kwargs):
         if unique and not indexed:
             raise ValueError('A unique property must be indexed.')
+        if auto and auto_default == NOT_PROVIDED:
+            raise ValueError('Properties with auto=True should also set an '
+                             'auto_default.')
         self.indexed = self.db_index = indexed
         self.indexed_fulltext = indexed_fulltext
         self.indexed_range = indexed_range
+        self.indexed_by_member = indexed_by_member
         self.has_own_index = has_own_index
         self.unique = unique
         self.editable = editable
         self.blank = blank
         self.null = null
         self.serialize = serialize
+        self.auto = auto
+        self.auto_default = auto_default
         self.meta = metadata
         self._default = default
 
@@ -86,31 +95,25 @@ class Property(object):
             return force_unicode(self._default, strings_only=True)
         return None
 
-    @classmethod
-    def to_neo(cls, value):
+    def to_neo(self, value):
         return value
 
-    @classmethod
-    def from_python(cls, value):
+    def from_python(self, value):
         """
         A python-centric alias for to_neo()
         """
-        return cls.to_neo(value)
+        return self.to_neo(value)
 
-    @classmethod
-    def from_neo(cls, value):
+    def from_neo(self, value):
         return value
 
-    @classmethod
-    def to_python(cls, value):
+    def to_python(self, value):
         """
         A python-centric alias for from_neo()
         """
-        return cls.from_neo(value)
+        return self.from_neo(value)
 
-
-    @classmethod
-    def to_neo_index(cls, value):
+    def to_neo_index(self, value):
         """
         Convert a Python value to how it should be represented in a Neo4j
         index - often, a string. Subclasses that wish to provide indexing
@@ -121,7 +124,7 @@ class Property(object):
         as how they should be returned by an ascending range query. Properties
         that don't support said option need not be concerned.
         """
-        return cls.to_neo(value)
+        return self.to_neo(value)
 
     def contribute_to_class(self, cls, name):
         """
@@ -214,9 +217,11 @@ class BoundProperty(AttrRouter):
                      'db_index',
                      'indexed_fulltext',
                      'indexed_range',
+                     'indexed_by_member',
                      'unique',
                      'to_neo',
                      'to_neo_index',
+                     'member_to_neo_index',
                      'from_python',
                      'from_neo',
                      'to_python',
@@ -228,6 +233,10 @@ class BoundProperty(AttrRouter):
                      'run_validators',
                      'pre_save',
                      'serialize',
+                     'auto',
+                     'auto_default',
+                     'next_value',
+                     'next_value_gremlin',
                      'meta',
                      'MAX',
                      'MIN',
@@ -240,6 +249,7 @@ class BoundProperty(AttrRouter):
         properties[self.name] = self # XXX: weakref
 
     attname = name = property(lambda self: self.__attname)
+    target = property(lambda self: self.__class)
 
     def _property_type(self):
         return type(self._property)
@@ -250,17 +260,17 @@ class BoundProperty(AttrRouter):
     @staticmethod
     def __values_of(instance, create=True):
         try:
-            values = instance.__values
-            for key, prop in BoundProperty._all_properties_for(instance).items(): #XXX: Might be a faster/more elegant way
-                if hasattr(prop._property, 'auto_now'):
-                    if prop._property.auto_now and prop.__attname not in values:
-                        values[prop.__attname] = datetime.datetime.now() #XXX:Setting to None here sets the node's datetime to None
-                        if type(prop._property) == DateProperty: #XXX:Kinda gross way to handle it :\
-                            values[prop.__attname] = datetime.datetime.date(values[prop.__attname])
+            values = instance._prop_values
+            #for key, prop in BoundProperty._all_properties_for(instance).items(): #XXX: Might be a faster/more elegant way
+            #    if prop.attname not in values:
+            #        if getattr(prop._property, 'auto_now', False):
+            #            values[prop.__attname] = datetime.datetime.now() #XXX:Setting to None here sets the node's datetime to None
+            #            if type(prop._property) == DateProperty: #XXX:Kinda gross way to handle it :\
+            #                values[prop.__attname] = datetime.datetime.date(values[prop.__attname])
         except:
             values = {}
             if create:
-                instance.__values = values
+                instance._prop_values = values
         return values
 
     @staticmethod
@@ -286,23 +296,61 @@ class BoundProperty(AttrRouter):
         return new_property_dict
 
     def index(self, using):
-        if not self.indexed:
+        if not (self.indexed or self.auto):
             raise TypeError("'%s' is not indexed" % (self.__propname,))
         else:
             return self.__class.index(using)
 
-    def _save_(instance, node, node_is_new):
+    def _save_(instance, node, node_is_new): #TODO this entire method could be transactional
         values = BoundProperty.__values_of(instance)
-        if values:
-            properties = BoundProperty._all_properties_for(instance)
-            for key, value in values.items():
-                prop = properties[key]
+        properties = BoundProperty._all_properties_for(instance)
+        for key, prop in properties.items():
+            index = None
+            if prop.auto and values.get(key, None) is None:
+                type_node = prop.target._type_node(instance.using)
+                
+                last_auto_attname = '%s.%s' % (prop.attname, AUTO_PROP_INDEX_VALUE)
+
+                script = prop.next_value_gremlin
+                script += \
+                """
+                typeNode = g.v(typeNodeID)
+                rawTypeNode = typeNode.getRawVertex()
+
+                lockManager.getWriteLock(rawTypeNode)
+
+                if (lastAutoProp in typeNode.map) {
+                    value = typeNode[lastAutoProp]
+                    value = nextValue(value)
+                }
+                else {
+                    value = defaultAutoVal
+                }
+                typeNode[lastAutoProp] = value
+
+                lockManager.releaseWriteLock(rawTypeNode, null)
+                
+                results = value
+                """
+                conn = connections[instance.using]
+                value = conn.gremlin_tx_deadlock_proof(script, 0,
+                      defaultAutoVal=prop.auto_default, lastAutoProp=last_auto_attname,
+                      typeNodeID=type_node.id)
+                #if the code is interrupted between here and setting
+                #the value, there might be a problem... worst case, lost
+                #id space?
+
+                values[key] = value
+            if key in values:
+                value = values[key]
                 value = prop.pre_save(node, node_is_new, prop.name) or value
+                values[key] = value
                 old, value = prop.__set_value(instance, value)
                 if prop.indexed:
+                    index = index if index else prop.index(using=instance.using)
                     if prop.unique:#TODO empty values? in validators.empty? # and value is not None:
                         try:
-                            old_node = prop.index(using=instance.using)[prop.attname][value]
+                            old_node = index[prop.attname][value]
                         except NotFoundError, e:
                             old_node = None
                         if old_node and old_node != node:
@@ -311,10 +359,12 @@ class BoundProperty(AttrRouter):
                                 (instance.__class__.__name__,
                                     prop.name))
                     if old is not None:
-                        prop.index(using=instance.using).remove(old, node)
+                        index.delete(prop.attname, None, node)
                     if value is not None:
-                        prop.index(using=instance.using).add(prop.attname, prop.to_neo_index(value), node)
-            values.clear()
+                        index.add(prop.attname, prop.to_neo_index(value), node)
+                        if prop.indexed_by_member:
+                            for m in value:
+                                index.add(prop.attname, prop.member_to_neo_index(m), node)
     NodeModel._save_properties = staticmethod(_save_) #TODO this needs to be revised. I hope there's a better way.
     del _save_
 
@@ -341,7 +391,9 @@ class BoundProperty(AttrRouter):
             pass
         else:
             try:
-                return self._property.from_neo(underlying[self.__propname])
+                values = BoundProperty.__values_of(instance)
+                values[self.__propname] = val = self._property.from_neo(underlying[self.__propname])
+                return val
             except: # no value set on node
                 pass
         return self.get_default() # fall through: default value
@@ -355,7 +407,6 @@ class BoundProperty(AttrRouter):
 
     @transactional
     def __set_value(self, instance, value):
-
         underlying = getattr(instance, 'node', None) or \
                 getattr(instance, 'relationship', None)
         if not underlying:
@@ -366,7 +417,7 @@ class BoundProperty(AttrRouter):
             old = None
         self._property.clean(value, instance)
         #supports null properties
-        if not value is None:
+        if not value in validators.EMPTY_VALUES:
             #should already have errored if self.null==False
             value = self._property.to_neo(value)
             underlying[self.__propname] = value
@@ -395,7 +446,6 @@ class StringProperty(Property):
         if min_length is not None:
             self.validators.append(validators.MinLengthValidator(min_length))
 
-    @classmethod
     def to_neo(cls, value):
         return unicode(value)
 
@@ -431,8 +481,7 @@ class IntegerProperty(Property):
     def to_neo(self, value):
         return int(value)
 
-    @classmethod
-    def to_neo_index(cls, value):
+    def to_neo_index(self, value):
         #for now, we'll just use a fixed-width binary decimal encoding with a
         #'-' for negative and '0' for positive or 0.
         s = str(abs(value))
@@ -440,6 +489,31 @@ class IntegerProperty(Property):
             raise ValueError('Values should be between {0} and {1}.'.format(MIN_INT, MAX_INT))
         return ('-' if value < 0 else '0') + s.zfill(19)
 
+class AutoProperty(IntegerProperty):
+    def __init__(self, *args, **kwargs):
+        kwargs['auto'] = True
+        kwargs['auto_default'] = 1
+        super(AutoProperty, self).__init__(*args, **kwargs)
+
+    def get_default(self):
+        return None
+
+    def next_value(self, old_value):
+        return old_value + 1
+
+    @property
+    def next_value_gremlin(self):
+        """
+        Return a Gremlin/Groovy closure that can compute next_value()
+        server-side. The function should be named 'nextValue', and take a
+        single value as an argument to increment.
+        """
+        script = \
+        """
+        def nextValue = { i -> i + 1}
+        """
+        return script
+
 class DateProperty(Property):
     __format = '%Y-%m-%d'
 
@@ -490,8 +564,7 @@ class DateProperty(Property):
                      .replace('%m', str(value.month).zfill(2))\
                      .replace('%d', str(value.day).zfill(2))
 
-    @classmethod
-    def from_neo(cls, value):
+    def from_neo(self, value):
         if value is None or value == '':
             return None
         if isinstance(value, datetime.datetime):
@@ -499,10 +572,9 @@ class DateProperty(Property):
         if isinstance(value, datetime.date):
             return value
 
-        return cls.__parse_date_string(value)
+        return self.__parse_date_string(value)
 
-    @classmethod
-    def to_neo(cls, value):
+    def to_neo(self, value):
         result = None
 
         if value is None:
@@ -512,9 +584,9 @@ class DateProperty(Property):
         elif isinstance(value, datetime.date):
             result = value
         else:
-            result = cls.__parse_date_string(value)
+            result = self.__parse_date_string(value)
 
-        return cls._format_date(result)
+        return self._format_date(result)
 
     def pre_save(self, model_instance, add, attname):
         if self.auto_now or (self.auto_now_add and add):
@@ -560,8 +632,7 @@ class DateTimeProperty(DateProperty):
 
         return result
 
-    @classmethod
-    def from_neo(cls, value):
+    def from_neo(self, value):
         if value is None or value == '':
             return None
 
@@ -570,10 +641,9 @@ class DateTimeProperty(DateProperty):
         if isinstance(value, datetime.date):
             return datetime.datetime(value.year, value.month, value.day)
 
-        return cls.__parse_datetime_string(value)
+        return self.__parse_datetime_string(value)
 
-    @classmethod
-    def to_neo(cls, value):
+    def to_neo(self, value):
         result = None
 
         if value is None:
@@ -583,9 +653,9 @@ class DateTimeProperty(DateProperty):
         elif isinstance(value, datetime.date):
             result = datetime.datetime(value.year, value.month, value.day)
         else:
-            result = cls.__parse_datetime_string(value)
+            result = self.__parse_datetime_string(value)
 
-        return cls._format_datetime(result)
+        return self._format_datetime(result)
 
     def pre_save(self, model_instance, add, attname):
         if self.auto_now or (self.auto_now_add and add):
@@ -600,6 +670,8 @@ class ArrayProperty(Property):
 
     default_validators = [validate_array]
 
+    member_to_neo_index = Property.to_neo_index.im_func
+
     def __init__(self, *args, **kwargs):
         """
         Keyword arguments:
@@ -607,6 +679,9 @@ class ArrayProperty(Property):
             of the sequence, or a tuple containing a list of validators and an
             error message, in that order.
         """
+        if kwargs.get('indexed', False):
+            if 'indexed_by_member' not in kwargs:
+                kwargs['indexed_by_member'] = True
         super(ArrayProperty, self).__init__(*args, **kwargs)
         per_key = 'per_element_validators'
         if per_key in kwargs:
@@ -618,6 +693,15 @@ class ArrayProperty(Property):
                 el_val = ElementValidator(vals_or_tuple)
             self.validators.append(el_val)
 
+    def get_default(self):
+        return []
+
+    #def from_neo(self, value):
+    #    from nose.tools import set_trace; set_trace()
+    #    if value is None:
+    #        return []
+    #    return value
+
 class StringArrayProperty(ArrayProperty):
     default_validators = [validate_str_array]
 
@@ -633,3 +717,5 @@ class URLArrayProperty(StringArrayProperty):
 
 class IntArrayProperty(ArrayProperty):
     default_validators = [validate_int_array]
+
+    member_to_neo_index = IntegerProperty.to_neo_index.im_func
