@@ -1,7 +1,7 @@
 def setup():
-    global Person, gdb
+    global Person, gdb, models
 
-    from neo4django.tests import Person, neo4django, gdb
+    from neo4django.tests import Person, gdb, models
 
 def teardown():
     gdb.cleandb()
@@ -18,3 +18,4 @@ def test_json_serialize():
     dave = Person()
     dave.save()
     assert json_serializer.serialize(Person.objects.all())
+
