@@ -1,4 +1,4 @@
-from nose.tools import eq_
+from nose.tools import eq_, with_setup
 
 def setup():
     global Person, neo4django, settings, gdb, models
@@ -24,11 +24,18 @@ def test_basic_relationship():
     lifesWork = RelatedPaper()
     lifesWork.save()
     lifesWork.authors.add(sandra)
+    
     lifesWork.save()
     work = list(sandra.papers.all())
     assert lifesWork in work, "Paper not found in %s" % repr(work)
     authors = list(lifesWork.authors.all())
     assert sandra in authors, "Author not found in %s" % repr(work)
+    #find all shared neo4j relationships
+    sandras = sandra.node.relationships.all(['OWNED_BY'])[:]
+    eq_(len(sandras), 1)
+    #test proper direction
+    eq_(sandras[0].end, sandra.node)
+    eq_(sandras[0].start, lifesWork.node)
 
 def test_basic_relationship_manager():
     class SomeOtherPaper(models.NodeModel):
@@ -36,7 +43,6 @@ def test_basic_relationship_manager():
                 rel_type = neo4django.Outgoing.OWNED_BY,
                 related_name = 'papers'
             )
-    #from nose.tools import set_trace; set_trace()
     pete = Person.objects.create(name="PETE!")
     boring_paper = SomeOtherPaper()
     boring_paper.authors.add(pete)
@@ -236,7 +242,10 @@ def test_rel_self():
 
 def test_rel_string_target():
     class Child(models.NodeModel):
-        parents = models.Relationship('neo4django.Person', 'CHILD_OF')
+        parents = models.Relationship('neo4django.Person',
+                                      neo4django.Outgoing.CHILD_OF)
+
+    assert 'child_set' in (f.name for f in Person._meta.fields)
 
     child = Child()
     child.parents.add(Person.objects.create(name='Han'))
@@ -244,3 +253,67 @@ def test_rel_string_target():
     child.save()
 
     eq_(('Han','Leia'), tuple(sorted(p.name for p in child.parents.all())))
+
+def test_rel_string_type():
+    class Child1(models.NodeModel):
+        parents = models.Relationship(Person, 'CHILD_OF')
+
+    child = Child1()
+    child.parents.add(Person.objects.create(name='Han'))
+    child.parents.add(Person.objects.create(name='Leia'))
+    child.save()
+
+    eq_(('Han','Leia'), tuple(sorted(p.name for p in child.parents.all())))
+
+    childs = child.node.relationships.all(['CHILD_OF'])[:]
+    eq_(len(childs), 2)
+    #test proper direction
+    for r in childs:
+        eq_(r.start, child.node)
+
+@with_setup(None, teardown)
+def test_abstract_rel_inheritance():
+    """
+    Test that inheriting abstract relationships doesn't throw an error. Stems
+    from GitHub issue #37.
+    """
+    class ZenNode(models.NodeModel):
+        class Meta:
+            abstract = True
+        rel = models.Relationship('self',rel_type='knows')
+
+    class Pupil(ZenNode):
+        pass
+
+    p = Pupil.objects.create()
+    p.rel.add(p)
+    p.save()
+
+@with_setup(None, teardown)
+def test_rel_query_direction():
+    """
+    Confirm GitHub issue #42, querying doesn't respect rel direction.
+    """
+    class LetterL(models.NodeModel):
+        name = models.StringProperty()
+
+    class LetterM(models.NodeModel):
+        name = models.StringProperty()
+        follows = models.Relationship(LetterL, rel_type='follows')
+
+    class LetterN(models.NodeModel):
+        name = models.StringProperty()
+        follows = models.Relationship(LetterM, rel_type='follows')
+
+    el = LetterL.objects.create(name='LLL')
+
+    m = LetterM.objects.create(name='MMM')
+    m.follows.add(el)
+    m.save()
+
+    n = LetterN.objects.create(name='NNN')
+    n.follows.add(m)
+    n.save()
+
+    eq_(len(list(m.follows.all())), 1)
+    eq_(len(list(m.lettern_set.all())), 1)
