@@ -6,4 +6,8 @@ NEO4J_DATABASES = {
     }
 }
 
+DEBUG = True
+
+NEO4DJANGO_PROFILE_REQUESTS = False
+
 NEO4J_DELETE_KEY = 'supersecretdebugkey!'
